@@ -1405,22 +1405,6 @@ PeerConnectionObserver.prototype = {
 
   handleIceConnectionStateChange: function(iceConnectionState) {
     let pc = this._dompc;
-    if (pc.iceConnectionState === 'new') {
-      var checking_histogram = Services.telemetry.getHistogramById("WEBRTC_ICE_CHECKING_RATE");
-      if (iceConnectionState === 'checking') {
-        checking_histogram.add(true);
-      } else if (iceConnectionState === 'failed') {
-        checking_histogram.add(false);
-      }
-    } else if (pc.iceConnectionState === 'checking') {
-      var success_histogram = Services.telemetry.getHistogramById("WEBRTC_ICE_SUCCESS_RATE");
-      if (iceConnectionState === 'completed' ||
-          iceConnectionState === 'connected') {
-        success_histogram.add(true);
-      } else if (iceConnectionState === 'failed') {
-        success_histogram.add(false);
-      }
-    }
 
     if (iceConnectionState === 'failed') {
       pc.logError("ICE failed, see about:webrtc for more details");
