@@ -52,7 +52,6 @@ mailing address.
 #include "gfxPlatform.h"
 #include "qcms.h"
 #include <algorithm>
-#include "mozilla/Telemetry.h"
 
 using namespace mozilla::gfx;
 
@@ -1088,11 +1087,5 @@ nsGIFDecoder2::SkipSubBlocks(const char* aData)
                                   nextSubBlockLength);
 }
 
-Maybe<Telemetry::ID>
-nsGIFDecoder2::SpeedHistogram() const
-{
-  return Some(Telemetry::IMAGE_DECODE_SPEED_GIF);
-}
-
 } // namespace image
 } // namespace mozilla
