@@ -1,5 +1,4 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* vim:set ts=4 sw=4 et cindent: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -931,7 +930,6 @@ nsresult
 nsSocketTransport::InitWithConnectedSocket(PRFileDesc *fd, const NetAddr *addr)
 {
     MOZ_ASSERT(PR_GetCurrentThread() == gSocketThread, "wrong thread");
-    NS_ASSERTION(!mFD.IsInitialized(), "already initialized");
 
     char buf[kNetAddrMaxCStrBufSize];
     NetAddrToString(addr, buf, sizeof(buf));
@@ -957,6 +955,7 @@ nsSocketTransport::InitWithConnectedSocket(PRFileDesc *fd, const NetAddr *addr)
     {
         MutexAutoLock lock(mLock);
 
+        NS_ASSERTION(!mFD.IsInitialized(), "already initialized");
         mFD = fd;
         mFDref = 1;
         mFDconnected = 1;
@@ -1320,11 +1319,14 @@ nsSocketTransport::InitiateSocket()
     //
     // if we already have a connected socket, then just attach and return.
     //
-    if (mFD.IsInitialized()) {
+    {
+      MutexAutoLock lock(mLock);
+      if (mFD.IsInitialized()) {
         rv = mSocketTransportService->AttachSocket(mFD, this);
         if (NS_SUCCEEDED(rv))
-            mAttached = true;
+          mAttached = true;
         return rv;
+      }
     }
 
     //
@@ -1390,18 +1392,18 @@ nsSocketTransport::InitiateSocket()
     PR_SetSocketOption(fd, &opt);
 #endif
 
-    // inform socket transport about this newly created socket...
-    rv = mSocketTransportService->AttachSocket(fd, this);
-    if (NS_FAILED(rv)) {
-        CloseSocket(fd);
-        return rv;
-    }
-    mAttached = true;
-
     // assign mFD so that we can properly handle OnSocketDetached before we've
     // established a connection.
     {
         MutexAutoLock lock(mLock);
+        // inform socket transport about this newly created socket...
+        rv = mSocketTransportService->AttachSocket(fd, this);
+        if (NS_FAILED(rv)) {
+            CloseSocket(fd);
+            return rv;
+        }
+        mAttached = true;
+
         mFD = fd;
         mFDref = 1;
         mFDconnected = false;
@@ -1546,8 +1548,12 @@ nsSocketTransport::RecoverFromError()
 
     nsresult rv;
 
-    // OK to check this outside mLock
-    NS_ASSERTION(!mFDconnected, "socket should not be connected");
+#ifdef DEBUG
+    {
+      MutexAutoLock lock(mLock);
+      NS_ASSERTION(!mFDconnected, "socket should not be connected");
+    }
+#endif
 
     // all connection failures need to be reported to DNS so that the next
     // time we will use a different address if available.
@@ -2099,7 +2105,7 @@ NS_IMETHODIMP
 nsSocketTransport::OpenInputStream(uint32_t flags,
                                    uint32_t segsize,
                                    uint32_t segcount,
-                                   nsIInputStream **result)
+                                   nsIInputStream **aResult)
 {
     SOCKET_LOG(("nsSocketTransport::OpenInputStream [this=%p flags=%x]\n",
         this, flags));
@@ -2108,6 +2114,7 @@ nsSocketTransport::OpenInputStream(uint32_t flags,
 
     nsresult rv;
     nsCOMPtr<nsIAsyncInputStream> pipeIn;
+    nsCOMPtr<nsIInputStream> result;
 
     if (!(flags & OPEN_UNBUFFERED) || (flags & OPEN_BLOCKING)) {
         // XXX if the caller wants blocking, then the caller also gets buffered!
@@ -2127,18 +2134,20 @@ nsSocketTransport::OpenInputStream(uint32_t flags,
                           NS_ASYNCCOPY_VIA_WRITESEGMENTS, segsize);
         if (NS_FAILED(rv)) return rv;
 
-        *result = pipeIn;
+        result = pipeIn;
     }
     else
-        *result = &mInput;
+        result = &mInput;
 
     // flag input stream as open
     mInputClosed = false;
 
     rv = PostEvent(MSG_ENSURE_CONNECT);
-    if (NS_FAILED(rv)) return rv;
+    if (NS_FAILED(rv)) {
+        return rv;
+    }
 
-    NS_ADDREF(*result);
+    result.forget(aResult);
     return NS_OK;
 }
 
@@ -2146,7 +2155,7 @@ NS_IMETHODIMP
 nsSocketTransport::OpenOutputStream(uint32_t flags,
                                     uint32_t segsize,
                                     uint32_t segcount,
-                                    nsIOutputStream **result)
+                                    nsIOutputStream **aResult)
 {
     SOCKET_LOG(("nsSocketTransport::OpenOutputStream [this=%p flags=%x]\n",
         this, flags));
@@ -2155,6 +2164,8 @@ nsSocketTransport::OpenOutputStream(uint32_t flags,
 
     nsresult rv;
     nsCOMPtr<nsIAsyncOutputStream> pipeOut;
+    nsCOMPtr<nsIOutputStream> result;
+
     if (!(flags & OPEN_UNBUFFERED) || (flags & OPEN_BLOCKING)) {
         // XXX if the caller wants blocking, then the caller also gets buffered!
         //bool openBuffered = !(flags & OPEN_UNBUFFERED);
@@ -2173,10 +2184,10 @@ nsSocketTransport::OpenOutputStream(uint32_t flags,
                           NS_ASYNCCOPY_VIA_READSEGMENTS, segsize);
         if (NS_FAILED(rv)) return rv;
 
-        *result = pipeOut;
+        result = pipeOut;
     }
     else
-        *result = &mOutput;
+        result = &mOutput;
 
     // flag output stream as open
     mOutputClosed = false;
@@ -2184,7 +2195,7 @@ nsSocketTransport::OpenOutputStream(uint32_t flags,
     rv = PostEvent(MSG_ENSURE_CONNECT);
     if (NS_FAILED(rv)) return rv;
 
-    NS_ADDREF(*result);
+    result.forget(aResult);
     return NS_OK;
 }
 
@@ -3035,6 +3046,17 @@ nsSocketTransport::PRFileDescAutoLock::SetKeepaliveVals(bool aEnabled,
     }
     return NS_OK;
 
+#elif defined(XP_DARWIN)
+    // Darwin uses sec; only supports idle time being set.
+    int err = setsockopt(sock, IPPROTO_TCP, TCP_KEEPALIVE,
+                         &aIdleTime, sizeof(aIdleTime));
+    if (NS_WARN_IF(err)) {
+        LogOSError("nsSocketTransport Failed setting TCP_KEEPALIVE",
+                   mSocketTransport);
+        return NS_ERROR_UNEXPECTED;
+    }
+    return NS_OK;
+
 #elif defined(XP_UNIX)
     // Not all *nix OSes support the following setsockopt() options
     // build errors will tell us if they are not.
